'****************************************************************
'*  Name    : 24LC256_H.inc                                     *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet 2016]              *
'*          : All Rights Reserved                               *
'*  Date    : 08/03/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   : Driver for the I2C 24LC256 eeprom memory          *
'*          : using the hardware I2C of PIC(r). MSSP MASTER MODE*
'* Compiler : Proton Basic version 3575                         *
'****************************************************************
'
' 10/03/2016 Version 1.1 Fixed error in $Define _24LC256_ for the file.
' Changed by $define _24LC256_inc
'
' 13/03/2016 Version 1.2
' Changed the name of the _HRSOutVersionFile_ to _HRSOutVersionFile_inc_
' Added configuration of the I2C bus pins to digital for the PIC18F2xF22 series.
'
'-------------------------------------------------------------------------------
'===============================================================================
' This program is free software, you can redistribute it and/or modify.
' The code is offered as-is and the author take any responsibility for its use.
' This program is distributed in the hope that it will be useful, but WITHOUT ANY
' WARRANTY. However, a mention of the author and compiler would be appreciated.
'
' Este programa es libre, puede redistribuirlo y/o modificarlo.
' Se entrega este cdigo tal como esta y el autor NO se hace responsable de
' su utilizacin. Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.

' Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' Ce logiciel est offert tel qu'il est, et son auteur NE prend AUCUNE responsabilit
' pour son utilization. Ce programme est distribu dans l'espoir qu'il sera util,
' mais SANS AUCUNE GARANTIE. Cependant mentionner l'auteur et le compileur serait
' aprci.
'===============================================================================
' LIBRARY COMMANDS:
' This is a libary using the Hardware I2C commands HBusOut/HBusIn.
' This library allows to read and write to 24C256 32KB external eeprom memory, using
' individual byte or Page read/write of 8 and 64 bytes.
' All routines are compatible with the PIC18F25K20 and PIC18F25K22 for the Amicus18 board.
' For others PICs please check datatsheet.
'
'===============================================================================
'-------------------------------------------------------------------------------       
'*******************************************************************************
$ifndef _24LC256_inc_
    $define _24LC256_inc_
'-------------------------------------------------------------------------------
    GoTo Over24LC256_H
'*******************************************************************************    
'-------------------------------------------------------------------------------
' Definition of the parameters of the I2C Bus.
' Input        : 
' Output       : 
' Notes        : 

#ifnSym __SDA_Port
    Declare SDA_Pin PORTC.4          ' SDA connected to this pin
#endIfSym
#ifnSym __SCL_Port
    Declare SCL_Pin PORTC.3          ' SCL connected to this pin
    Declare Hbus_Bitrate 400           ' Bus frequency 100kHz for the TC74 sensor 
'    Declare Slow_Bus On   
#endIfSym
'-------------------------------------------------------------------------------
' Uncomment the next line to disable the NACK function.
'$define _DisableNACK_

' Uncomment the next line to enable the interrupt protection.
'$define _EnableGIE_Save_

' Uncomment the next line to work with the 24LC512 eeprom memory.
'$define _24LC512_

'-------------------------------------------------------------------------------
'*******************************************************************************
' Declare a variable for INTCON copy.
    $ifndef _xINTCON_Save_Var_
    $define _xINTCON_Save_Var_
    Dim xINTCON_Save As Byte System
    $endif

' This new command save and clear the GIE global interrupt.
$ifndef GIE_Save
    $define GIE_Save()          '              
    #ifdef __INTERRUPTS_ENABLED '
    xINTCON_Save = INTCON       '
    INTCONbits_GIE = 0          '
    #endif                      '

$endif

'-------------------------------------------------------------------------------    
' This new command restore the GIE global interrupt.
$ifndef GIE_Restore
    $define GIE_Restore()       '
    #ifdef __INTERRUPTS_ENABLED '
    If xINTCON_Save.7 = 1 Then  '
        INTCONbits_GIE = 1      '
    EndIf                       '
    #endif                      '
    
$endif

'===============================================================================
' Add local (non-persistent) byte variables here
$ifndef _LVBS_Block1_
    $define _LVBS_Block1_
    Dim LVSByte1 As Byte System
    Dim LVSByte2 As Byte System
    Dim LVSByte3 As Byte System
    Dim LVSByte4 As Byte System
$endif

' Add local (non-persistent) Words variables here
$ifndef _LVWS_Block1_
    $define _LVWS_Block1_
    Dim LVSWord1 As Word System     ' Local Variable System Word1
    Dim LVSWord2 As Word System     ' Local Variable System Word2
$endif

$ifndef _LVWS_Block2_
    $define _LVWS_Block2_
    Dim LVSWord3 As Word System     ' Local Variable System Word3
    Dim LVSWord4 As Word System     ' Local Variable System Word4
$endif

' Add more variables/aliases here.
'    Dim E2P256AddressWrite As Word System
'    Dim E2P256AddressRead  As Word System
    Dim E2P256AddressWrite As LVSWord3
    Dim E2P256AddressRead  As LVSWord4

    Dim NumberBytesPage As Byte System
'-------------------------------------------------------------------------------
' Changing the memory type it is possible to drive a 512 Kbits device. (24C512)
' Example: to change to the 512 device. Write theses lines before the include file.
' uncomment the line to use the code for the 24C512 device.
'    $define _24C512_

    $if (!_defined(_24LC256_) && !_defined(_24LC512_))
    $define _24LC256_
    $endif

' Define the number of pages for the 24C256.
' eeprom memory: 32768 / 8  = 4096 pages of 8 bits
' eeprom memory: 32768 / 64 =  512 pages of 64 bits
'
' Define the number of pages for the 24C512.
' eeprom memory: 65536 / 8   = 8192 pages of 8 bits
' eeprom memory: 65536 / 128 =  512 pages of 64 bits
'
' Alias for FSR0 word
    $ifndef _FSR0_Var_
    $define _FSR0_Var_
    Dim FSR0 As FSR0L.Word
    $endif

'===============================================================================
' Write one data byte to the 24C256 eeeprom.
' Input : none
' Output: Return Var = NACK (optional)
' Syntax: WriteByte_E2P(Number of Device(0-7), Address(Word), Data(Byte), Return NACK(Byte)) 
' Notes : 
'
    Dim yE2PAddress   As LVSWord1   ' Local Word
    Dim yNumberDevice As LVSByte1   ' Local Byte
    Dim yData         As LVSByte2   ' Local Byte
    Dim yACKReturn    As LVSByte3   ' Local Byte 

$define WriteByte_E2P(pNbDevice, pAddress, pData, pACKReturn) _mWriteByte_24C256 pNbDevice, pAddress, pData, pACKReturn

_mWriteByte_24C256 Macro- pNbDevice, pAddress, pData, pACKReturn
    #if (Prm_Count > 4)
	   #error "_mWriteByte_24C256 - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 3)
            #error "_mWriteByte_24C256 - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mWriteByte_24C256 - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_1 == Num8)
                    Num_Byte pNbDevice, yNumberDevice                    
                #endif
                #if(Prm_1 == Byte)
                    Byte_Byte pNbDevice, yNumberDevice                    
                #endif
            #endif
            #if (Prm_2 != Num8) && (Prm_2 != Num16) && (Prm_2 != Byte) && (Prm_2 !=Word)
                #error "_mWriteByte_24C256 - (Param2) Should be a Byte or Num8"
                Exitm
            #else
                #if ((Prm_2 == Num8) || (Prm_2 == Num16))
                    Num_Word pAddress, yE2PAddress                    
                #endif
                #if (Prm_2 == Byte)
                    Byte_Word pAddress, yE2PAddress                    
                #endif
                #if (Prm_2 == Word)
                    Word_Word pAddress, yE2PAddress                    
                #endif
            #endif
            #if(Prm_3 != Num8) && (Prm_3 != Byte)
                #error "_mWriteByte_24C256 - (Param3) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_3 == Num8)
                    Num_Byte pData, yData                    
                #endif
                #if(Prm_3 == Byte)
                    Byte_Byte pData, yData                    
                #endif
            #endif
            GoSub _mWriteByte_24C256_Sub
            ' Return the NACK
            $ifndef _DisableNACK_
            #if (Prm_Count == 4)
                #if (Prm_4 != Byte)
                    #error "_mWriteByte_24C256 - (Param4) Must be a Byte!"
                    Exitm
                #else
                    Byte_Byte yACKReturn, pACKReturn
                #endif
            #endif
            $endif    
        #endif
    #endif                
Endm
    
#ifMacro- _mWriteByte_24C256
_mWriteByte_24C256_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    $ifndef _DisableNACK_
    yACKReturn = 0                  ' Set the NACK flag error 
    $endif
    GoSub E2P256Device_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    HBStart
    HBusOut E2P256AddressWrite      ' Address and Write  
    '---------------------------------------------------------------------------
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then           ' (ACKSTAT) Check ACK from the sensor
        yACKReturn = 1              ' Set A NACK error
        GoTo WriteByte_24C256Exit  
    EndIf
    $endif
    $ifdef _SSP1CON2    
    If SSP1CON2.6 = 1 Then          ' (ACKSTAT) Check ACK from the sensor
        yACKReturn = 1              ' Set A NACK error
        GoTo WriteByte_24C256Exit
    EndIf
    $endif
    $endif    
    '---------------------------------------------------------------------------
    HBusOut yE2PAddress.HighByte    ' Point to the e2p address.
    HBusOut yE2PAddress.LowByte
    HBusOut yData
WriteByte_24C256Exit:
    HBStop
    '---------------------------------------------------------------------------    
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    DelayMS 5                       ' Delay to write 
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Read one data byte from the 24C256 eeeprom.
' Input : none
' Output: Return Var = NACK (optional)
' Syntax: ReadByte_E2P(Number of Device(0-7), Address(Word), Data(Byte), Return NACK(Byte)) 
' Notes : 
'
'    Dim yE2PAddress   As LVSWord1
'    Dim yNumberDevice As LVSByte1
'    Dim yData         As LVSByte2
'    Dim yACKReturn    As LVSByte3  

$define ReadByte_E2P(pNbDevice, pAddress, pACKReturn) _mReadByte_24C256 pNbDevice, pAddress, pACKReturn

_mReadByte_24C256 Macro- pNbDevice, pAddress, pACKReturn
    #if (Prm_Count > 3)
	   #error "_mReadByte_24C256 - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 2)
            #error "_mReadByte_24C256 - Too few parameters"
            Exitm
        #else
            #if (Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mWriteByte_24C256 - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if (Prm_1 == Num8)
                    Num_Byte pNbDevice, yNumberDevice                    
                #endif
                #if (Prm_1 == Byte)
                    Byte_Byte pNbDevice, yNumberDevice                    
                #endif
            #endif
            #if (Prm_2 != Num8) && (Prm_2 != Num16) && (Prm_2 != Byte) && (Prm_2 !=Word)
                #error "_mReadByte_24C256 - (Param2) Should be a Byte or Num8"
                Exitm
            #else
                #if ((Prm_2 == Num8) || (Prm_2 == Num16))
                    Num_Word pAddress, yE2PAddress                    
                #endif
                #if (Prm_2 == Byte)
                    Byte_Word pAddress, yE2PAddress                    
                #endif
                #if (Prm_2 == Word)
                    Word_Word pAddress, yE2PAddress                    
                #endif
            #endif
            GoSub _mReadByte_24C256_Sub

            #if (_mReadByte_24C256_RETURN != 1)
                #error "_mReadByte_24C256 - Mandatory return parameter misssing"
                Exitm
            #else
                #if(Return_Type != Byte)
                    #error "_mReadByte_24C256 - Return variable should be a Byte variable"
                    Exitm
                #else
                    Byte_Byte yData, Return_Var
                #endif
            #endif     
            $ifndef _DisableNACK_
            #if (Prm_Count == 3)
                #if (Prm_3 != Byte)
                    #error "_mReadByte_24C256 - (Param3) Must be a Byte!"
                    Exitm
                #else
                    Byte_Byte yACKReturn, pACKReturn
                #endif
            #endif    
            $endif
        #endif
    #endif                
Endm
    
#ifMacro- _mReadByte_24C256
_mReadByte_24C256_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    $ifndef _DisableNACK_
    yACKReturn = 0                  ' Set the NACK flag error 
    $endif
    GoSub E2P256Device_Sub         ' get the e2p address
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    HBStart
    HBusOut E2P256AddressWrite      ' Address and Write  
    '---------------------------------------------------------------------------
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then           ' (ACKSTAT) Check ACK from the sensor
        yACKReturn = 1              ' Set A NACK error
        GoTo ReadByte_24C256Exit  
    EndIf
    $endif
    $ifdef _SSP1CON2    
    If SSP1CON2.6 = 1 Then          ' (ACKSTAT) Check ACK from the sensor
        yACKReturn = 1              ' Set A NACK error
        GoTo ReadByte_24C256Exit
    EndIf
    $endif
    $endif    
    '---------------------------------------------------------------------------
    HBusOut yE2PAddress.HighByte    ' Point to the e2p address.
    HBusOut yE2PAddress.LowByte
    HBReStart
    HBusOut E2P256AddressRead       ' Address and Read
    HBusIn yData
    HBusNack
ReadByte_24C256Exit:
    HBStop
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------    
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Write a page of data bytes to the 24C256/24C512 eeeprom.
' Input : none
' Output: Return Var = NACK (optional)
' Syntax: WritePage_E2P8(Number of Device(Byte), ArrayName(Input), Number page(Byte), Return NACK(Byte)) 
' Notes : The datas are taken from an array
'
'    Dim yE2PAddress   As LVSWord1
'    Dim yNumberDevice As LVSByte1
'    Dim yData         As LVSByte2
'    Dim yACKReturn    As LVSByte3
    Dim yNbBytes       As LVSByte4 ' Local Byte
    Dim yNumberPage    As LVSWord2 ' Local Word

$define WritePage_E2P8(pNbDevice, pArrayInput, pNbPage, pACKReturn) '
    NumberBytesPage = 8                                             '
_mWritePage_24C256 pNbDevice, pArrayInput, pNbPage, pACKReturn

$ifdef _24LC256_
$define WritePage_E2P64(pNbDevice, pArrayInput, pNbPage, pACKReturn) '
    NumberBytesPage = 64                                             '
_mWritePage_24C256 pNbDevice, pArrayInput, pNbPage, pACKReturn

$endif

$ifdef _24LC512_
$define WritePage_E2P128(pNbDevice, pArrayInput, pNbPage, pACKReturn) '
    NumberBytesPage = 128                                             '
_mWritePage_24C256 pNbDevice, pArrayInput, pNbPage, pACKReturn

$endif

_mWritePage_24C256 Macro- pNbDevice, pArrayInput, pNbPage, pACKReturn
    #if (Prm_Count > 4)
	   #error "_mWritePage_24C256 - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 3)
            #error "_mWritePage_24C256 - Too few parameters"
            Exitm
        #else
            #if (Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mWritePage_24C256 - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if (Prm_1 == Num8)
                    Num_Byte pNbDevice, yNumberDevice                    
                #endif
                #if (Prm_1 == Byte)
                    Byte_Byte pNbDevice, yNumberDevice                    
                #endif
            #endif
            #if ((Prm_2 != String) && (Prm_2 != Byte_Array_Ptr))
                #error "_mWritePage_24C256 (Param2) must be a String or a byte array"
                Exitm
            #else
                Num_FSR0 pArrayInput
            #endif
            #if (Prm_3 != Num8) && (Prm_3 != Byte) && (Prm_3 != Num16) && (Prm_3 != Word)
                #error "_mWritePage_24C256 - (Param3) Should be a Byte, Word, Num8 or Num16"
                Exitm
            #else
                #if ((Prm_3 == Num8) || (Prm_3 == Num16))
                    Num_Word pNbPage, yNumberPage                    
                #endif
                #if(Prm_3 == Byte)
                    Byte_Word pNbPage, yNumberPage                    
                #endif
                #if(Prm_3 == Word)
                    Word_Word pNbPage, yNumberPage                    
                #endif
            #endif
            GoSub _mWritePage_24C256_Sub
            $ifndef _DisableNACK_
            #if (Prm_Count == 4)
                #if (Prm_4 != Byte)
                    #error "_mWritePage_24C256 - (Param4) Must be a Byte!"
                    Exitm
                #else
                    Byte_Byte yACKReturn, pACKReturn
                #endif
            #endif    
            $endif
        #endif
    #endif                
Endm

#ifMacro- _mWritePage_24C256
_mWritePage_24C256_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    $ifndef _DisableNACK_
    yACKReturn = 0                          ' Set the NACK flag error 
    $endif
    GoSub E2P256ConfigAddressDevice_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    HBStart
    HBusOut E2P256AddressWrite      ' Address and Write  
    '---------------------------------------------------------------------------
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then           ' (ACKSTAT) Check ACK from the sensor
        yACKReturn = 1              ' Set A NACK error
        GoTo WritePage_24C256Exit
    EndIf
    $endif
    $ifdef _SSP1CON2    
    If SSP1CON2.6 = 1 Then          ' (ACKSTAT) Check ACK from the sensor
        yACKReturn = 1              ' Set A NACK error
        GoTo WritePage_24C256Exit 
    EndIf
    $endif
    $endif    
    '---------------------------------------------------------------------------
    HBusOut yE2PAddress.HighByte    ' Point to the e2p address.
    HBusOut yE2PAddress.LowByte
    Repeat            
        HBusOut POSTINC0            ' POSTINC0 increment automaticaly the FSR0
        Dec yNbBytes
    Until yNbBytes = 0              ' Until all bytes are written.
WritePage_24C256Exit:
    HBStop
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------    
    DelayMS 5                       ' Delay to write    
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Read a page of  data bytes from the 24C256/24C512 eeeprom.
' Input : none
' Output: Array
' Syntax: ReadPage_E2P8(Number of Device(Byte), ArrayName(Output), Number page(Byte), Return NACK(Byte)) 
' Notes : The datas are saved into an array.
'
'    Dim yE2PAddress   As LVSWord1
'    Dim yNumberDevice As LVSByte1
'    Dim yData         As LVSByte2
'    Dim yACKReturn    As LVSByte3
'    Dim yNbBytes      As LVSByte4
'    Dim yNumberPage  As LVSWord2

$define ReadPage_E2P8(pNbDevice, pArrayOutput, pNbPage, pACKReturn) '
    NumberBytesPage = 8                                             '
_mReadPage_24C256 pNbDevice, pArrayOutput, pNbPage, pACKReturn

$ifdef _24LC256_
$define ReadPage_E2P64(pNbDevice, pArrayOutput, pNbPage, pACKReturn) '
    NumberBytesPage = 64                                             '
_mReadPage_24C256 pNbDevice, pArrayOutput, pNbPage, pACKReturn

$endif

$ifdef _24LC512_
$define ReadPage_E2P128(pNbDevice, pArrayOutput, pNbPage, pACKReturn) '
    NumberBytesPage = 128                                             '
_mReadPage_24C256 pNbDevice, pArrayOutput, pNbPage, pACKReturn

$endif

_mReadPage_24C256 Macro- pNbDevice, pArrayOutput, pNbPage, pACKReturn
    #if (Prm_Count > 4)
	   #error "_mReadPage_24C256 - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 3)
            #error "_mReadPage_24C256 - Too few parameters"
            Exitm
        #else
            #if (Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mReadPage_24C256 - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if (Prm_1 == Num8)
                    Num_Byte pNbDevice, yNumberDevice                    
                #endif
                #if (Prm_1 == Byte)
                    Byte_Byte pNbDevice, yNumberDevice                    
                #endif
            #endif
            #if ((Prm_2 != String) && (Prm_2 != Byte_Array_Ptr))
                #error "_mReadPage_24C256 (Param2) must be a String or a byte array"
                Exitm
            #else
                Num_FSR0 pArrayOutput
            #endif
            #if (Prm_3 != Num8) && (Prm_3 != Byte) && (Prm_3 != Num16) && (Prm_3 != Word)
                #error "_mReadPage_24C256 - (Param3) Should be a Byte, Word, Num8 or Num16"
                Exitm
            #else
                #if ((Prm_3 == Num8) || (Prm_3 == Num16))
                    Num_Word pNbPage, yNumberPage                    
                #endif
                #if(Prm_3 == Byte)
                    Byte_Word pNbPage, yNumberPage                    
                #endif
                #if(Prm_3 == Word)
                    Word_Word pNbPage, yNumberPage                    
                #endif
            #endif
            GoSub _mReadPage_24C256_Sub
            $ifndef _DisableNACK_
            #if (Prm_Count == 4)
                #if (Prm_4 != Byte)
                    #error "_mReadPage_24C256 - (Param4) Must be a Byte!"
                    Exitm
                #else
                    Byte_Byte yACKReturn, pACKReturn
                #endif
            #endif
            $endif    
        #endif
    #endif                
Endm

#ifMacro- _mReadPage_24C256
_mReadPage_24C256_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    $ifndef _DisableNACK_
    yACKReturn = 0                  ' Set the NACK flag error 
    $endif
    GoSub E2P256ConfigAddressDevice_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    HBStart
    HBusOut E2P256AddressWrite      ' Address and Write  
    '---------------------------------------------------------------------------
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then           ' (ACKSTAT) Check ACK from the sensor
        yACKReturn = 1              ' Set A NACK error
        GoTo ReadPage_24C256Exit 
    EndIf
    $endif
    $ifdef _SSP1CON2    
    If SSP1CON2.6 = 1 Then          ' (ACKSTAT) Check ACK from the sensor
        yACKReturn = 1              ' Set A NACK error
        GoTo ReadPage_24C256Exit 
    EndIf
    $endif    
    $endif
    '---------------------------------------------------------------------------
    HBusOut yE2PAddress.HighByte    ' Point to the e2p address.
    HBusOut yE2PAddress.LowByte
    HBReStart
    HBusOut E2P256AddressRead       ' Address and Read
ReadPage_24C256AA_Sub:            
    HBusIn POSTINC0                 ' Read & Save, POSTINC0 increment automaticaly the FSR0
    Dec yNbBytes
    If yNbBytes > 0 Then
        HBusAck
        GoTo ReadPage_24C256AA_Sub
    EndIf
    HBusNack
ReadPage_24C256Exit:
    HBStop
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------    
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Detect if the 24C256 eeeprom is present in the I2C bus.
' Input : none
' Output: Return_Byte ACKReturn
' Syntax: M24LC256_Present(NumberOfDevice(Byte, ACKReturn(Byte)) 
' Notes : The answer must be = 0. If ACKReturn > 0 then there is a NACK error.
'
$ifndef _DisableNACK_
$define M24LC256_Present(pNbDevice, pACKReturn) yNbBytes = ReadByte_E2P(pNbDevice, 0, pACKReturn)

$define M24LC512_Present(pNbDevice, pACKReturn) yNbBytes = ReadByte_E2P(pNbDevice, 0, pACKReturn)

$else
$define M24LC256_Present(pNbDevice, pACKReturn) '
    WriteByte_E2P(pNbDevice, 0, $A5)            '
    DelayMS 5                                   '
    yNbBytes = ReadByte_E2P(pNbDevice, 0)       '
    pACKReturn = yNbBytes ^ $A5

$define M24LC512_Present(pNbDevice, pACKReturn) '
    WriteByte_E2P(pNbDevice, 0, $A5)            '
    DelayMS 5                                   '
    yNbBytes = ReadByte_E2P(pNbDevice, 0)       '
    pACKReturn = yNbBytes ^ $A5
        
$endif
'-------------------------------------------------------------------------------
' Calculate the Address write and read of the 24LC256/24LC512.
' Input : 
' Output:
' Notes : 
'
' Define the number of pages for the 24C256.
' eeprom memory: 32768 / 8  = 4096 pages of 8 bits
' eeprom memory: 32768 / 64 = 512 pages of 64 bits
'
' Define the number of pages for the 24C512.
' eeprom memory: 65536 / 8   = 80192 pages of 8 bits
' eeprom memory: 65536 / 128 = 512 pages of 64 bits
'
E2P256ConfigAddressDevice_Sub:
    yE2PAddress = yNumberPage * NumberBytesPage  ' Address eeprom to write 
    yNbBytes = NumberBytesPage                   ' Number of bytes to write/read
'
E2P256Device_Sub:
    $ifdef _24LC256_
    yE2PAddress = yE2PAddress & $7FFF              ' No more than 32KBytes for the 24C256 eeprom.
    $endif
'
    E2P256AddressWrite = %10100000 | ((yNumberDevice & 7) << 1)     
    E2P256AddressRead = E2P256AddressWrite
    Inc E2P256AddressRead
    Return
'*******************************************************************************
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
' This Cdata line is compiled if the _HRSOutVersionFile_ is present only.
$ifdef _HRSOutVersionFile_inc_
_24LC256_inc_Version: CData "24LC256_H.inc Version 1.2 - 13/03/2016",0
$endif
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
Over24LC256_H:
'===============================================================================
' Set the I2C pins to digital. (Result = 2 lines)
$if (_device = _18F23K22) Or (_device = _18F24K22) Or (_device = _18F25K22) Or (_device = _18F26K22)
$ifndef _AnselI2CSoftPins_
$define _AnselI2CSoftPins_
#if (__SCL_PORT == PORTA)
    #if (__SCL_PIN == 0)
        $ifdef _ANSEL
        ANSEL.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSEL
        ANSEL.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSEL
        ANSEL.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SCL_PIN == 3)
        $ifdef _ANSEL
        ANSEL.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SCL_PIN == 5)
        $ifdef _ANSEL
        ANSEL.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SCL_PORT == PORTB)
    #if (__SCL_PIN == 0)
        $ifdef _ANSELH
        ANSELH.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSELH
        ANSELH.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSELH
        ANSELH.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SCL_PIN == 3)
        $ifdef _ANSELH
        ANSELH.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SCL_PIN == 4)
        $ifdef _ANSELH
        ANSELH.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SCL_PORT == PORTC)
    $ifdef _ANSELC
    #if (__SCL_PIN == 2)
        ANSELC.2 = 0
    #endif
    #if (__SCL_PIN == 3)
        ANSELC.3 = 0
    #endif
    #if (__SCL_PIN == 4)
        ANSELC.4 = 0
    #endif
    #if (__SCL_PIN == 5)
        ANSELC.5 = 0
    #endif
    #if (__SCL_PIN == 6)
        ANSELC.6 = 0
    #endif
    #if (__SCL_PIN == 7)
        ANSELC.7 = 0
    #endif
    $endif
#endif

$ifdef _PORTD
#if (__SCL_PORT == PORTD)
    $ifdef _ANSELD
    #if (__SCL_PIN == 0)
        ANSELD.0 = 0
    #endif
    #if (__SCL_PIN == 1)
        ANSELD.1 = 0
    #endif
    #if (__SCL_PIN == 2)
        ANSELD.2 = 0
    #endif
    #if (__SCL_PIN == 3)
        ANSELD.3 = 0
    #endif
    #if (__SCL_PIN == 4)
        ANSELD.4 = 0
    #endif
    #if (__SCL_PIN == 5)
        ANSELD.5 = 0
    #endif
    #if (__SCL_PIN == 6)
        ANSELD.6 = 0
    #endif
    #if (__SCL_PIN == 7)
        ANSELD.7 = 0
    #endif
    $endif    
#endif
$endif

$ifdef _PORTE
#if (__SCL_PORT == PORTE)
    #if (__SCL_PIN == 0)
        $ifdef _ANSEL       
        ANSEL.5 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSEL
        ANSEL.6 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSEL
        ANSEL.7 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.2 = 0
        $endif
    #endif
#endif
$endif
'-------------------------------------------------------------------------------
#if (__SDA_PORT == PORTA)
    #if (__SDA_PIN == 0)
        $ifdef _ANSEL
        ANSEL.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSEL
        ANSEL.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSEL
        ANSEL.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SDA_PIN == 3)
        $ifdef _ANSEL
        ANSEL.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SDA_PIN == 5)
        $ifdef _ANSEL
        ANSEL.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SDA_PORT == PORTB)
    #if (__SDA_PIN == 0)
        $ifdef _ANSELH
        ANSELH.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSELH
        ANSELH.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSELH
        ANSELH.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SDA_PIN == 3)
        $ifdef _ANSELH
        ANSELH.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SDA_PIN == 4)
        $ifdef _ANSELH
        ANSELH.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SDA_PORT == PORTC)
    $ifdef _ANSELC
    #if (__SDA_PIN == 2)
        ANSELC.2 = 0
    #endif
    #if (__SDA_PIN == 3)
        ANSELC.3 = 0
    #endif
    #if (__SDA_PIN == 4)
        ANSELC.4 = 0
    #endif
    #if (__SDA_PIN == 5)
        ANSELC.5 = 0
    #endif
    #if (__SDA_PIN == 6)
        ANSELC.6 = 0
    #endif
    #if (__SDA_PIN == 7)
        ANSELC.7 = 0
    #endif
    $endif
#endif

$ifdef _PORTD
#if (__SDA_PORT == PORTD)
    $ifdef _ANSELD
    #if (__SDA_PIN == 0)
        ANSELD.0 = 0
    #endif
    #if (__SDA_PIN == 1)
        ANSELD.1 = 0
    #endif
    #if (__SDA_PIN == 2)
        ANSELD.2 = 0
    #endif
    #if (__SDA_PIN == 3)
        ANSELD.3 = 0
    #endif
    #if (__SDA_PIN == 4)
        ANSELD.4 = 0
    #endif
    #if (__SDA_PIN == 5)
        ANSELD.5 = 0
    #endif
    #if (__SDA_PIN == 6)
        ANSELD.6 = 0
    #endif
    #if (__SDA_PIN == 7)
        ANSELD.7 = 0
    #endif
    $endif    
#endif
$endif

$ifdef _PORTE
#if (__SDA_PORT == PORTE)
    #if (__SDA_PIN == 0)
        $ifdef _ANSEL       
        ANSEL.5 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSEL
        ANSEL.6 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSEL
        ANSEL.7 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.2 = 0
        $endif
    #endif
#endif
$endif
$endif      ' _AnselI2CSoftPins_
$endif      ' Ifdevice
'-------------------------------------------------------------------------------
$endif
'-------------------------------------------------------------------------------

