'****************************************************************
'*  Name    : 24LC256_S.inc                                     *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet 2016]              *
'*          : All Rights Reserved                               *
'*  Date    : 08/03/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   : Driver for the I2C 24LC256 eeprom memory          *
'*          : using the software I2C of PIC(r).                 *
'* Compiler : Proton Basic version 3575                         *
'****************************************************************
'
' Version 1.1 10/03/2016  
' Fixed error in $Define _24LC256_ for the file.
' Changed by $Define _24LC256_inc
'
' Version 1.2 13/03/2016
' Changed the declaration of the I2C pins. Now It is possible to use the Declare Busin/BusOut pins.
'
'-------------------------------------------------------------------------------
'===============================================================================
' This program is free software, you can redistribute it and/or modify.
' The code is offered as-is and the author take any responsibility for its use.
' This program is distributed in the hope that it will be useful, but WITHOUT ANY
' WARRANTY. However, a mention of the author and compiler would be appreciated.
'
' Este programa es libre, puede redistribuirlo y/o modificarlo.
' Se entrega este cdigo tal como esta y el autor NO se hace responsable de
' su utilizacin. Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.

' Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' Ce logiciel est offert tel qu'il est, et son auteur NE prend AUCUNE responsabilit
' pour son utilization. Ce programme est distribu dans l'espoir qu'il sera util,
' mais SANS AUCUNE GARANTIE. Cependant mentionner l'auteur et le compileur serait
' aprci.
'===============================================================================
' LIBRARY COMMANDS:
' This is a libary using Software I2C commands BusOut/BusIn.
' This library allows to read and write to 24C256 32KB or 24C512 64KB external eeprom memory, using
' individual byte or Page read/write of 8 and 64/129 bytes.
' All routines are compatible with the PIC18F25K20 and PIC18F25K22 for the Amicus18 board.
' For others PICs please check the datatsheet.
'
'===============================================================================
'-------------------------------------------------------------------------------       
'*******************************************************************************
$ifndef _24LC256_inc_
    $define _24LC256_inc_
'-------------------------------------------------------------------------------
    GoTo Over24LC256_S2Exit
'*******************************************************************************    
'-------------------------------------------------------------------------------
' Definition of pins of the I2C Bus.
' Input Variables   : 
' Output Variables  : 
' Notes             : The pins direction could be changed by the user,
'                   : because this Driver uses all software code.
#ifnSym __SDA_Port
    Declare SDA_Pin PORTC.4           ' SDA connected to this pin
#endIfSym
#ifnSym __SCL_Port
    Declare SCL_Pin PORTC.3           ' SCL connected to this pin
#endIfSym
'-------------------------------------------------------------------------------
' Uncomment the next line to enable the interrupt protection.
'$define _EnableGIE_Save_

'*******************************************************************************
' Declare a variable for INTCON copy.
    $ifndef _xINTCON_Save_Var_
    $define _xINTCON_Save_Var_
    Dim xINTCON_Save As Byte System
    $endif

' This new command save and clear the GIE global interrupt.
$ifndef GIE_Save
    $define GIE_Save()          '              
    #ifdef __INTERRUPTS_ENABLED '
    xINTCON_Save = INTCON       '
    INTCONbits_GIE = 0          '
    #endif                      '

$endif

'-------------------------------------------------------------------------------    
' This new command restore the GIE global interrupt.
$ifndef GIE_Restore
    $define GIE_Restore()       '
    #ifdef __INTERRUPTS_ENABLED '
    If xINTCON_Save.7 = 1 Then  '
        INTCONbits_GIE = 1      '
    EndIf                       '
    #endif                      '
    
$endif

'===============================================================================
' Add local (non-persistent) byte variables here
$ifndef _LVBS_Block1_
    $define _LVBS_Block1_
    Dim LVSByte1 As Byte System
    Dim LVSByte2 As Byte System
    Dim LVSByte3 As Byte System
    Dim LVSByte4 As Byte System
$endif

' Add local (non-persistent) Words variables here
$ifndef _LVWS_Block1_
    $define _LVWS_Block1_
    Dim LVSWord1 As Word System     ' Local Variable System Word1
    Dim LVSWord2 As Word System     ' Local Variable System Word2
$endif

$ifndef _LVWS_Block2_
    $define _LVWS_Block2_
    Dim LVSWord3 As Word System     ' Local Variable System Word3
    Dim LVSWord4 As Word System     ' Local Variable System Word4
$endif

' Add more variables/aliases here.
    Dim E2P256AddressWrite As LVSWord3
    Dim E2P256AddressRead  As LVSWord4

    Dim NumberBytesPage As LVSByte1

' Alias for FSR0 word
    $ifndef _FSR0_Var_
    $define _FSR0_Var_
    Dim FSR0 As FSR0L.Word
    $endif
'-------------------------------------------------------------------------------
' Changing the memory type it is possible to drive a 512 Kbits device. (24C512)
' Example: to change to the 512 device. Write theses lines before the include file.
' uncomment the line to use the code for the 24C512 device.
'    $define _24C512_

    $if (!_defined(_24LC256_) && !_defined(_24LC512_))
    $define _24LC256_
    $endif

' Define the number of pages for the 24C256.
' eeprom memory: 32768 / 8  = 4096 pages of 8 bits
' eeprom memory: 32768 / 64 =  512 pages of 64 bits
'
' Define the number of pages for the 24C512.
' eeprom memory: 65536 / 8   = 8192 pages of 8 bits
' eeprom memory: 65536 / 128 =  512 pages of 64 bits
'
'===============================================================================
' Write one data byte to the 24C256 eeeprom.
' Input : none
' Output: Return Var = NACK (optional)
' Syntax: WriteByte_E2P(Number Device(Byte), Address(Word), Data(Byte)) 
' Notes : 
'
    Dim yE2PAddress   As LVSWord1   ' Local Word
    Dim yNumberDevice As LVSByte2   ' Local Byte
    Dim yData         As LVSByte3   ' Local Byte

$define WriteByte_E2P(pNbDevice, pAddress, pData) _mWriteByte_24C256 pNbDevice, pAddress, pData

_mWriteByte_24C256 Macro- pNbDevice, pAddress, pData
    #if (Prm_Count > 3)
	   #error "_mWriteByte_24C256 - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 3)
            #error "_mWriteByte_24C256 - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mWriteByte_24C256 - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_1 == Num8)
                    Num_Byte pNbDevice, yNumberDevice                    
                #endif
                #if(Prm_1 == Byte)
                    Byte_Byte pNbDevice, yNumberDevice                    
                #endif
            #endif
            #if (Prm_2 != Num8) && (Prm_2 != Num16) && (Prm_2 != Byte) && (Prm_2 !=Word)
                #error "_mWriteByte_24C256 - (Param2) Should be a Byte or Num8"
                Exitm
            #else
                #if ((Prm_2 == Num8) || (Prm_2 == Num16))
                    Num_Word pAddress, yE2PAddress                    
                #endif
                #if (Prm_2 == Byte)
                    Byte_Word pAddress, yE2PAddress                    
                #endif
                #if (Prm_2 == Word)
                    Word_Word pAddress, yE2PAddress                    
                #endif
            #endif
            #if(Prm_3 != Num8) && (Prm_3 != Byte)
                #error "_mWriteByte_24C256 - (Param3) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_3 == Num8)
                    Num_Byte pData, yData                    
                #endif
                #if(Prm_3 == Byte)
                    Byte_Byte pData, yData                    
                #endif
            #endif
            GoSub _mWriteByte_24C256_Sub
        #endif
    #endif                
Endm

#ifMacro- _mWriteByte_24C256
_mWriteByte_24C256_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    GoSub E2P256Device_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2COut __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, E2P256AddressWrite, yE2PAddress,[yData]
    '---------------------------------------------------------------------------    
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    DelayMS 5       ' Delay to write 
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Read one data byte from the 24C256 eeeprom.
' Input : none
' Output: Return Var = NACK (optional)
' Syntax: Result = ReadByte_E2P(Number Device(Byte), Address(Word)) 
' Notes : 
'
'    Dim yE2PAddress   As LVSWord1   ' Local Word
'    Dim yNumberDevice As LVSByte2   ' Local Byte
'    Dim yData         As LVSByte3   ' Local Byte

$define ReadByte_E2P(pNbDevice, pAddress) _mReadByte_24C256 pNbDevice, pAddress

_mReadByte_24C256 Macro- pNbDevice, pAddress
    #if (Prm_Count > 2)
	   #error "_mReadByte_24C256 - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 2)
            #error "_mReadByte_24C256 - Too few parameters"
            Exitm
        #else
            #if (Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mWriteByte_24C256 - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if (Prm_1 == Num8)
                    Num_Byte pNbDevice, yNumberDevice                    
                #endif
                #if (Prm_1 == Byte)
                    Byte_Byte pNbDevice, yNumberDevice                    
                #endif
            #endif
            #if (Prm_2 != Num8) && (Prm_2 != Num16) && (Prm_2 != Byte) && (Prm_2 !=Word)
                #error "_mReadByte_24C256 - (Param2) Should be a Byte or Num8"
                Exitm
            #else
                #if ((Prm_2 == Num8) || (Prm_2 == Num16))
                    Num_Word pAddress, yE2PAddress                    
                #endif
                #if (Prm_2 == Byte)
                    Byte_Word pAddress, yE2PAddress                    
                #endif
                #if (Prm_2 == Word)
                    Word_Word pAddress, yE2PAddress                    
                #endif
            #endif
            GoSub _mReadByte_24C256_Sub

            #if (_mReadByte_24C256_RETURN != 1)
                #error "_mReadByte_24C256 - Mandatory return parameter misssing"
                Exitm
            #else
                #if(Return_Type != Byte)
                    #error "_mReadByte_24C256 - Return variable should be a Byte variable"
                    Exitm
                #else
                    Byte_Byte yData, Return_Var
                #endif
            #endif     
        #endif
    #endif                
Endm

#ifMacro- _mReadByte_24C256
_mReadByte_24C256_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    GoSub E2P256Device_Sub         ' get the e2p address
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, E2P256AddressRead, yE2PAddress,[yData]
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------    
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Write a page of data bytes to the 24C256 eeeprom.
' Input : none
' Output: Return Var = NACK (optional)
' Syntax: WritePage_E2P8(Number of Device(Byte), ArrayName(Input), Number page(Byte)) 
' Notes : The datas are taken from an array
'
'    Dim yE2PAddress   As LVSWord1   ' Local Word
'    Dim yNumberDevice As LVSByte2   ' Local Byte
'    Dim yData         As LVSByte3   ' Local Byte
    Dim yNumberPage    As LVSWord2 ' Local Word

$define WritePage_E2P8(pNbDevice, pArrayInput, pNbPage) '
    yNumberDevice = pNbDevice                           '
    yNumberPage = pNbPage                               '
    NumberBytesPage = 8                                 '
    GoSub E2P256ConfigAddressDevice_Sub                 '
    I2COut __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, E2P256AddressWrite, yE2PAddress,[Str pArrayInput\8] '
    DelayMS 5

'-------------------------------------------------------------------------------

$ifdef _24LC256_
$define WritePage_E2P64(pNbDevice, pArrayInput, pNbPage) '
    yNumberDevice = pNbDevice                            '
    yNumberPage = pNbPage                                '
    NumberBytesPage = 64                                 '
    GoSub E2P256ConfigAddressDevice_Sub                  '
    I2COut __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, E2P256AddressWrite, yE2PAddress,[Str pArrayInput\64] '
    DelayMS 5

$endif
'-------------------------------------------------------------------------------

$ifdef _24LC512_
$define WritePage_E2P128(pNbDevice, pArrayInput, pNbPage) '
    yNumberDevice = pNbDevice                             '
    yNumberPage = pNbPage                                 '
    NumberBytesPage = 128                                 '
    GoSub E2P256ConfigAddressDevice_Sub                   '
    I2COut __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, E2P256AddressWrite, yE2PAddress,[Str pArrayInput\128] '
    DelayMS 5
    
$endif
'-------------------------------------------------------------------------------
' Read a page of  data bytes from the 24C256 eeeprom.
' Input : none
' Output: Array
' Syntax: ReadPage_E2P8(Number of Device(Byte), ArrayName(Output), Number page(Byte), Return NACK(Byte)) 
' Notes : The datas are saved into an array.
'
'    Dim yE2PAddress   As LVSWord1   ' Local Word
'    Dim yNumberDevice As LVSByte2   ' Local Byte
'    Dim yData         As LVSByte3   ' Local Byte
'    Dim yNumberPage  As LVSWord2    ' Local Word

$define ReadPage_E2P8(pNbDevice, pArrayOutput, pNbPage) '
    yNumberDevice = pNbDevice                           '
    yNumberPage = pNbPage                               '
    NumberBytesPage = 8                                 '
    GoSub E2P256ConfigAddressDevice_Sub                 '
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, E2P256AddressRead, yE2PAddress,[Str pArrayOutput\8]

$ifdef _24LC256_
$define ReadPage_E2P64(pNbDevice, pArrayOutput, pNbPage) '
    yNumberDevice = pNbDevice                            '
    yNumberPage = pNbPage                                '
    NumberBytesPage = 64                                 '
    GoSub E2P256ConfigAddressDevice_Sub                  '
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, E2P256AddressRead, yE2PAddress,[Str pArrayOutput\64]

$endif

$ifdef _24LC512_
$define ReadPage_E2P128(pNbDevice, pArrayOutput, pNbPage) '
    yNumberDevice = pNbDevice                             '
    yNumberPage = pNbPage                                 '
    NumberBytesPage = 128                                 '
    GoSub E2P256ConfigAddressDevice_Sub                   '
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, E2P256AddressRead, yE2PAddress,[Str pArrayOutput\128]

$endif

'-------------------------------------------------------------------------------
' Detect if the 24C256 eeeprom is present in the I2C bus.
' Input : none
' Output: Return_Byte ACKReturn
' Syntax: M24LC256_Present(NumberOfDevice(Byte, ACKReturn(Byte)) 
' Notes : The answer must be = 0. If ACKReturn > 0 then there is a NACK error.
'
$ifdef _24LC256_
$define M24LC256_Present(pNbDevice, pACKReturn) '
    WriteByte_E2P(pNbDevice, 0, $A5)            '
    DelayMS 6                                   '
    yData = ReadByte_E2P(pNbDevice, 0)          '
    pACKReturn = yData ^ $A5

$endif

$ifdef _24LC512_
$define M24LC512_Present(pNbDevice, pACKReturn) '
    WriteByte_E2P(pNbDevice, 0, $A5)            '
    DelayMS 6                                   '
    yData = ReadByte_E2P(pNbDevice, 0)          '
    pACKReturn = yData ^ $A5

$endif   
'-------------------------------------------------------------------------------
' Calculate the Address write and read of the 24LC256/24LC512.
' Input : 
' Output:
' Notes : 
'
' Define the number of pages for the 24C256.
' eeprom memory: 32768 / 8  = 4096 pages of 8 bits
' eeprom memory: 32768 / 64 = 512 pages of 64 bits
'
' Define the number of pages for the 24C512.
' eeprom memory: 65536 / 8   = 80192 pages of 8 bits
' eeprom memory: 65536 / 128 = 512 pages of 64 bits
'
E2P256ConfigAddressDevice_Sub:
    yE2PAddress = yNumberPage * NumberBytesPage     ' Address eeprom to write 
'
E2P256Device_Sub:
    $ifdef _24LC256_
    yE2PAddress = yE2PAddress & $7FFF               ' No more than 32KBytes for the 24C256 eeprom.
    $endif
'
    E2P256AddressWrite = %10100000 | ((yNumberDevice & 7) << 1)     
    E2P256AddressRead = E2P256AddressWrite
    Inc E2P256AddressRead
    Return
'*******************************************************************************
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
$ifdef _HRSOutVersionFile_inc_
_24LC256_inc_Version: CData "24LC256_S2.inc Version 1.2 - 13/03/2016",0
$endif
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
Over24LC256_S2Exit:
$endif
'-------------------------------------------------------------------------------
